"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const WebSocket = require("isomorphic-ws");
const events_1 = require("events");
const CONNECTION_TIMEOUT = 2000;
const ACKNOWLEDGMENT_TIMEOUT = 2000;
const RESPONSE_TIMEOUT = 3000;
const timeout = async (ms, message) => new Promise((_, reject) => {
    const id = setTimeout(() => {
        clearTimeout(id);
        reject(new Error(message !== null && message !== void 0 ? message : `Timed out in ${ms}ms.`));
    }, ms);
});
const defer = () => {
    let resolve;
    let reject;
    const promise = new Promise((_resolve, _reject) => {
        resolve = _resolve;
        reject = _reject;
    });
    return { promise, resolve, reject };
};
const messageIsNotification = (input) => !!((input.id === undefined || input.id === null) && input.method);
class WSChannel {
    constructor(url) {
        this.isAlive = false;
        this._requestCounter = 0;
        this._pendingRequests = {};
        this._url = url;
        this._emitter = new events_1.EventEmitter();
    }
    async connect() {
        this._ws = new WebSocket(this._url);
        const connect = new Promise(resolve => {
            var _a;
            (_a = this._ws) === null || _a === void 0 ? void 0 : _a.on('open', () => {
                this.isAlive = true;
                resolve();
            });
        });
        const error = new Promise((_, reject) => {
            var _a;
            (_a = this._ws) === null || _a === void 0 ? void 0 : _a.on('error', err => {
                this.isAlive = false;
                reject(err);
            });
        });
        await Promise.race([
            connect,
            error,
            timeout(CONNECTION_TIMEOUT, `Could not connect in ${CONNECTION_TIMEOUT}ms`),
        ]);
        this._ws.on('ping', () => {
            this.isAlive = true;
        });
        this._ws.on('message', data => {
            this._handleMessage(data);
        });
    }
    async disconnect() {
        this._requestCounter = 0;
        this._pendingRequests = {};
        if (!this._ws) {
            return Promise.resolve();
        }
        return new Promise(resolve => {
            var _a, _b;
            (_a = this._ws) === null || _a === void 0 ? void 0 : _a.on('close', () => {
                var _a;
                (_a = this._ws) === null || _a === void 0 ? void 0 : _a.terminate();
                this.isAlive = false;
                this._ws = undefined;
                resolve();
            });
            (_b = this._ws) === null || _b === void 0 ? void 0 : _b.close();
        });
    }
    async invoke(actionName, params) {
        const request = {
            jsonrpc: '2.0',
            id: this._requestCounter,
            method: actionName,
            params: params !== null && params !== void 0 ? params : {},
        };
        const send = new Promise((resolve, reject) => {
            var _a;
            (_a = this._ws) === null || _a === void 0 ? void 0 : _a.send(JSON.stringify(request), (err) => {
                if (err) {
                    return reject(err);
                }
                return resolve();
            });
        });
        await Promise.race([
            send,
            timeout(ACKNOWLEDGMENT_TIMEOUT, `Request is not acknowledged in ${ACKNOWLEDGMENT_TIMEOUT}ms`),
        ]);
        const response = defer();
        this._pendingRequests[this._requestCounter] = response;
        this._requestCounter += 1;
        return Promise.race([
            response.promise,
            timeout(RESPONSE_TIMEOUT, `Response not received in ${RESPONSE_TIMEOUT}ms`),
        ]);
    }
    subscribe(eventName, cb) {
        this._emitter.on(eventName, cb);
    }
    _handleMessage(message) {
        var _a;
        const res = JSON.parse(message);
        if (messageIsNotification(res)) {
            this._emitter.emit(res.method, res.params);
        }
        else {
            const id = typeof res.id === 'number' ? res.id : parseInt(res.id, 10);
            if (this._pendingRequests[id]) {
                if (res.error) {
                    this._pendingRequests[id].reject(new Error((_a = res.error.data) !== null && _a !== void 0 ? _a : res.error.data));
                }
                else {
                    this._pendingRequests[id].resolve(res.result);
                }
                delete this._pendingRequests[id];
            }
        }
    }
}
exports.WSChannel = WSChannel;
//# sourceMappingURL=ws_channel.js.map